﻿;==============================================================================================================================================================================
; 
;==============================================================================================================================================================================

Procedure OpenSystemDatabase()
  
  Protected FileHandle.i, DatabaseUpdate.s
  
  ; Create the database if it doesn't already exist.
  
  FileHandle.i = OpenFile(#PB_Any, Program\DatabaseName)
  
  ; 
  
  If FileHandle.i
    
    ; 
    
    CloseFile(FileHandle.i)
    
    ; 
    
    Program\DatabaseHandle = OpenDatabase(#PB_Any, Program\DatabaseName, #EmptyString, #EmptyString)
    
    ; 
    
    If Program\DatabaseHandle
      
      ; Turn on auto database vacuum
      
      If Not DatabaseUpdate(Program\DatabaseHandle, "PRAGMA auto_vacuum = on")
        
      EndIf
      
      ; Write the Recipes table out
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Recipes("
      
      DatabaseUpdate.s + "Recipetitle TEXT, "
      
      DatabaseUpdate.s + "Numberofservings TEXT, "
      
      DatabaseUpdate.s + "Recipeauthor TEXT, "
      
      DatabaseUpdate.s + "Categories TEXT, "
      
      DatabaseUpdate.s + "Subcategories TEXT, "
      
      DatabaseUpdate.s + "Preparationtime TEXT, "
      
      DatabaseUpdate.s + "Cookingtime TEXT, "
      
      DatabaseUpdate.s + "Difficulty TEXT, "
      
      DatabaseUpdate.s + "Recipeversion TEXT, "
      
      DatabaseUpdate.s + "Recipesource TEXT, "
      
      DatabaseUpdate.s + "Copyright TEXT, "
      
      DatabaseUpdate.s + "Reciperating TEXT, "
      
      DatabaseUpdate.s + "Importedfrom TEXT, "
      
      DatabaseUpdate.s + "Authorcomments TEXT, "
      
      DatabaseUpdate.s + "Instructions TEXT, "
      
      DatabaseUpdate.s + "Nutritionaldata TEXT, "
      
      DatabaseUpdate.s + "Othercomments TEXT, "
      
      DatabaseUpdate.s + "Deleted TEXT, "
      
      DatabaseUpdate.s + "Updated TEXT, "
      
      DatabaseUpdate.s + "Favourite TEXT, "
      
      DatabaseUpdate.s + "Locked TEXT, "
      
      DatabaseUpdate.s + "Recordid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s + "UNIQUE (Recipetitle, Instructions) ON CONFLICT FAIL)"
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
        
        ; Debug DatabaseUpdate.s  + "  --  "  + DatabaseError()
        
      EndIf
      
      ; Create the index on Recipes table
      
      DatabaseUpdate.s = "CREATE INDEX IF NOT EXISTS Recipesindex ON Recipes("
      
      DatabaseUpdate.s + "Recipetitle, "
      
      DatabaseUpdate.s + "Instructions)"
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
        
        ; Debug DatabaseUpdate.s  + "  --  "  + DatabaseError()
        
      EndIf
      
      ; Write the Pictures table out
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Pictures("
      
      DatabaseUpdate.s + "Pictureid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s + "Picture BLOB, "
      
      DatabaseUpdate.s + "Description TEXT, "
      
      DatabaseUpdate.s + "FileName TEXT, "
      
      DatabaseUpdate.s + "Recordid INTEGER, "
      
      DatabaseUpdate.s + "FOREIGN KEY(Recordid) REFERENCES Recipes(Recordid) ON DELETE CASCADE)"     
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
        
        ; Debug DatabaseUpdate.s  + "  --  "  + DatabaseError()
        
      EndIf
      
      ; Write the ingredients table out
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Ingredients("
      
      DatabaseUpdate.s + "Ingredientid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s + "Unit TEXT, "
      
      DatabaseUpdate.s + "Measure TEXT, "
      
      DatabaseUpdate.s + "Ingredient TEXT, "
      
      DatabaseUpdate.s + "Preparation TEXT, "
      
      DatabaseUpdate.s + "Lineorder TEXT, "
      
      DatabaseUpdate.s + "Recordid INTEGER, "
      
      DatabaseUpdate.s + "FOREIGN KEY(Recordid) REFERENCES Recipes(Recordid) ON DELETE CASCADE)"     
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
        
        Debug DatabaseUpdate.s  + "  --  "  + DatabaseError()
        
      EndIf
      
      ; 
      
    Else
      
      MessageRequester("Database open error", "Could not open database file in database mode.", #PB_MessageRequester_Ok)
      
    EndIf
    
    ; 
    
  Else
    
    ; 
    
    MessageRequester("Database open error", "Could not open or create raw database file.", #PB_MessageRequester_Ok)
    
    ; 
    
  EndIf
  
  ; 
  
EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 78
; FirstLine = 31
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP
; CompileSourceDirectory