﻿;==============================================================================================================================================================================
; Universal, recursive search engine used by many functions
;==============================================================================================================================================================================

Procedure ImportBigOvenFile(ImportFileName.s)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected.s CurrentFile.s, DatabaseQuery.s, YieldNumber.s, PictureFile1.s, CrbPicturePath.s
  
  Protected.s MeasureString.s
  
  Protected.i MDBDatabaseHandle.i, PictureFileIn.i, FileInSize.i
  
  ;------------------------------------------------------------------------------------------------
  ; Try to configure the MDB data source via ODBC
  ;------------------------------------------------------------------------------------------------
  
  If SQLConfigDataSource_(0, #ODBC_ADD_DSN, "Microsoft Access Driver (*.mdb)","Server=127.0.0.1; Description=MyDescription ;DSN=ConnectMDB;DBQ=" + ImportFileName.s + ";UID=;PWD=" + Program\CrbPassword + ";")
    
    ;----------------------------------------------------------------------------------------------
    ; Now try to open the current MDB database
    ;----------------------------------------------------------------------------------------------
    
    MDBDatabaseHandle.i = OpenDatabase(#PB_Any, "ConnectMDB", #EmptyString, #EmptyString, #PB_Database_ODBC)
    
    ;----------------------------------------------------------------------------------------------
    ; Check to see if the database failed to open
    ;----------------------------------------------------------------------------------------------
    
    If MDBDatabaseHandle.i  <> #DatabaseOpenFail
      
      ;--------------------------------------------------------------------------------------------
      ; Create a new list to hold the recipe ID's in
      ;--------------------------------------------------------------------------------------------
      
      NewList RecipeId.s()
      
      ;--------------------------------------------------------------------------------------------
      ; Create the ID retrieval query from the BigOven Access database
      ;--------------------------------------------------------------------------------------------
      
      DatabaseQuery.s = "SELECT RecipeID FROM Recipes"
      
      ;--------------------------------------------------------------------------------------------
      ; Get all the current recipe ID's from the current BigOven (MS-Access) MDB file
      ;--------------------------------------------------------------------------------------------
      
      If DatabaseQuery(MDBDatabaseHandle.i, DatabaseQuery.s) <> #DatabaseQueryFail
        
        While NextDatabaseRow(MDBDatabaseHandle.i)
          
          AddElement(RecipeId.s())
          
          RecipeId.s()  = GetDatabaseString(MDBDatabaseHandle.i, 0)
          
        Wend
        
        FinishDatabaseQuery(MDBDatabaseHandle.i)
        
      Else 
        
        Debug "Can't get RecipeID from BigOven CRB file: " + DatabaseError()
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Process all the current recipe ID's from the current MDB file
      ;--------------------------------------------------------------------------------------------
      
      If ListSize(RecipeId.s())
        
        ;------------------------------------------------------------------------------------------
        ; Iterate each RecipeID from the list and process the wanted recipe elements
        ;------------------------------------------------------------------------------------------
        
        ForEach RecipeId.s()
          
          DatabaseQuery.s = "SELECT Title, YieldText, Instructions, Source, "
          
          DatabaseQuery.s + "YieldNumber, PrimaryIngredient, Cuisine, "
          
          DatabaseQuery.s + "PictureFile1, Notes, AllCategoriesText "
          
          DatabaseQuery.s + "FROM Recipes WHERE RecipeID = " + RecipeId.s()  + #EmptyString
          
          If DatabaseQuery(MDBDatabaseHandle.i, DatabaseQuery.s) <> #DatabaseQueryFail
            
            While NextDatabaseRow(MDBDatabaseHandle.i) <> 0
              
              Recipe\Recipetitle.s          = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 0), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Recipetitle.s          = ProperCaseString(Recipe\Recipetitle.s) 
              
              Recipe\Numberofservings.s     = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 1), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Instructions.s         = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 2), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Recipesource.s         = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 3), "'", "''", #PB_String_NoCase, 1)
              
              YieldNumber.s                 = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 4), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Categories.s           = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 5), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Cuisine.s              = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 6), "'", "''", #PB_String_NoCase, 1)
              
              PictureFile1.s                = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 7), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\Othercomments.s        = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 8), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\SubCategories.s        = ReplaceString(GetDatabaseString(MDBDatabaseHandle.i, 9), "'", "''", #PB_String_NoCase, 1)
              
              Recipe\SubCategories.s        = ReplaceString(Recipe\SubCategories.s, "|", ",", #PB_String_NoCase, 1)
              
              Recipe\SubCategories.s        = Trim(Recipe\SubCategories.s, ",")
              
              Recipe\Updated.s              = FormatDate("%yyyy-%mm-%dd", Date())
              
            Wend
            
            ;--------------------------------------------------------------------------------------
            ; 
            ;--------------------------------------------------------------------------------------
            
            FinishDatabaseQuery(MDBDatabaseHandle.i)
            
            ;--------------------------------------------------------------------------------------
            ; Save the recipe item into the SQLite database
            ;--------------------------------------------------------------------------------------
            
            If PictureFile1.s <> #EmptyString
              
              CrbPicturePath.s = GetPathPart(ImportFileName.s) + "Pictures\" 
              
              If FileSize(CrbPicturePath.s + PictureFile1.s) <> -1
                
                AddElement(Recipe\PictureList())
                
                Recipe\PictureList()\PictureBuffer = AllocateMemory(#PictureBufferSize)
                
                If Recipe\PictureList()\PictureBuffer = #Null
                  
                  Debug "memory allocation error!"
                  
                EndIf
                
                Recipe\PictureList()\FileType         = GetExtensionPart(PictureFile1.s)
                
                Recipe\PictureList()\FileName         = PictureFile1.s
                
                Recipe\PictureList()\Description      = #EmptyString
                
                Recipe\PictureList()\OriginalFileName = #EmptyString
                
                If Len(Recipe\PictureList()\OriginalFileName) = 0                 ; If the filename came back empty
                  
                  If Len(Recipe\PictureList()\FileName) = 0
                    
                    Recipe\PictureList()\FileName = "Dummy." + Recipe\PictureList()\FileType
                    
                  EndIf
                  
                  Recipe\PictureList()\OriginalFileName = Recipe\Recipetitle + "." + GetExtensionPart(Recipe\PictureList()\FileName)    ; Add the recipe title as the filename
                  
                EndIf
                
                PictureFileIn.i = ReadFile(#PB_Any, CrbPicturePath.s  + PictureFile1.s)
                
                If PictureFileIn.i
                  
                  FileInSize.i = Lof(PictureFileIn.i)
                  
                  If FileInSize.i <> 0
                    
                    If Recipe\PictureList()\PictureBuffer
                      
                      If ReadData(PictureFileIn.i, Recipe\PictureList()\PictureBuffer, FileInSize.i) = FileInSize.i
                        
                        Recipe\PictureList()\PictureBufferPtr = FileInSize.i
                        
                      EndIf
                      
                    EndIf
                    
                  Else
                    
                    Debug "We got no picture data for " + CrbPicturePath.s  + PictureFile1.s
                    
                  EndIf
                  
                  CloseFile(PictureFileIn.i)
                  
                Else
                  
                  Debug "We did not get a handle for "  + CrbPicturePath.s  + PictureFile1.s
                  
                EndIf
                
              Else
                
                Debug "The recipe picture file could not be found at: " + CrbPicturePath.s
                
              EndIf
              
            Else
              
              ; Debug "There was no picture name for this recipe"
              
            EndIf
            
            ;--------------------------------------------------------------------------------------
            ; Process the ingredients separately for this recipe
            ;--------------------------------------------------------------------------------------
            
            DatabaseQuery.s = "SELECT TextQty, Measure, Name, PrepNotes, LineOrder "
            
            DatabaseQuery.s + "FROM IngredientLines "
            
            DatabaseQuery.s + "WHERE RecipeID = "    + RecipeId.s()  + #EmptyString
            
            If DatabaseQuery(MDBDatabaseHandle.i, DatabaseQuery.s) <> #DatabaseQueryFail
              
              While NextDatabaseRow(MDBDatabaseHandle.i)
                
                AddElement(Recipe\IngredientList())
                
                Recipe\IngredientList()\Unit.s        = KillQuote(GetDatabaseString(MDBDatabaseHandle.i, 0))
                
                Recipe\IngredientList()\Measure.s     = KillQuote(GetDatabaseString(MDBDatabaseHandle.i, 1))
                
                Recipe\IngredientList()\Measure.s     = Trim(Recipe\IngredientList()\Measure.s)
                
                Recipe\IngredientList()\Measure.s     = CorrectIngredientMeasure(Recipe\IngredientList()\Measure)  ; Correct single letter ingredient measure strings
                
                Recipe\IngredientList()\Ingredient.s  = KillQuote(GetDatabaseString(MDBDatabaseHandle.i, 2))
                
                Recipe\IngredientList()\Preparation.s = KillQuote(GetDatabaseString(MDBDatabaseHandle.i, 3))
                
                Recipe\IngredientList()\LineOrder.s   = KillQuote(GetDatabaseString(MDBDatabaseHandle.i, 4))
                
              Wend
              
              FinishDatabaseQuery(MDBDatabaseHandle.i)
              
            EndIf
            
            ;--------------------------------------------------------------------------------------
            ; 
            ;--------------------------------------------------------------------------------------
            
          Else 
            
            Debug  "Could not get the current recipe from BigOven (MS-Access) MDB file: " + #CRLF$ + Recipe\Recipetitle.s + #CRLF$ + ImportFileName.s + #CRLF$ + "=============================="
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          SaveImportRecipe(ImportFileName.s)
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          ClearRecipeStructure()
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        Next ;RecipeID()  
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      Else
        
        ; The list is empty, nothing to process
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      CloseDatabase(MDBDatabaseHandle.i)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      SQLConfigDataSource_(0, #ODBC_REMOVE_DSN, "Microsoft Access Driver (*.mdb)","Server=127.0.0.1; Description=MyDescription ;DSN=ConnectMDB;DBQ=" + CurrentFile.s + ";UID=;PWD=" + Program\CrbPassword + ";")
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      Debug  "No BigOven (MS-Access) MDB database handle: " + DatabaseError()
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    Debug  "Could not configure the BigOven (MS-Access) MDB data source: " + DatabaseError()
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 234
; FirstLine = 203
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP