﻿;==============================================================================================================================================================================
; Import the Food Data Exchange XML data
;==============================================================================================================================================================================

Procedure ImportFoodDataExchangeCompressedFile_XML(ImportFileName.s, DataFileName.s, *CurrentNode, CurrentSublevel.i)

  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected NodeName.s, *ChildNode, AttributeName.s, FilenameString.s, PictureFileName.s, *Buffer
  
  Protected FileSize.i, PictureFileHandle.i, FileNamesCountLoop.i, FileNamePart.s, FDXZPicturePath.s
  
  Protected FileExtensionPart.s, Separator.s, FileIn.i, FileInSize.i, PictureFileIn.i
  
  Protected AttributeValue.s, NodeText.s
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    NodeName.s = GetXMLNodeName(*CurrentNode)
    
    ;----------------------------------------------------------------------------------------------
    ; Debug NodeName.s
    ;----------------------------------------------------------------------------------------------
    
    If NodeName = "hixz"
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s   = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s  = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s

            Case "Source"
              
              Recipe\Importedfrom.s   =  AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "Recipe"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Len(Recipe\Recipetitle)
        
        If Len(Recipe\Instructions.s)
          
          Recipe\Instructions.s = ReplaceString(Recipe\Instructions.s, "'", "''")
          
        EndIf
        
        SaveImportRecipe(ImportFileName.s)  ; Save the filled fields of the recipe structure
        
        ClearRecipeStructure()
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s               = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s              = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          ;Debug AttributeName
          
          Select AttributeName.s
              
            Case "Name"
              
              Recipe\Recipetitle.s      = AttributeValue.s
              
              Recipe\Recipetitle.s      = ProperCaseString(Recipe\Recipetitle.s)
              
            Case "Servings"
              
              Recipe\Numberofservings.s = AttributeValue.s
              
            Case "RecipeTypes"
              
              Recipe\Categories         = RemoveString(AttributeValue.s, " ")
              
            Case "Source"
              
              Recipe\Recipesource       = Trim(AttributeValue.s)
              
            Case "Comments"
              
              Recipe\Authorcomments.s   = AttributeValue.s
              
            Case "CookingTime"
              
              Recipe\Cookingtime.s      = AttributeValue.s
              
            Case "PreparationTime"
              
              Recipe\Preparationtime.s  = AttributeValue.s
              
            Case "Copyright"
              
              Recipe\Copyright.s        = AttributeValue.s
              
            Case "CreateDate"
              
              Recipe\Updated.s          = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Gather all the ingredient data here
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeIngredient"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      AddElement(Recipe\IngredientList())
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s         = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s        = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s
              
            Case "Quantity"
              
              Recipe\IngredientList()\Unit = AttributeValue.s
              
            Case "Unit"
              
              Recipe\IngredientList()\Measure = CorrectIngredientMeasure(AttributeValue.s)  ; Correct single letter ingredient measure strings
              
            Case "Ingredient"
              
              Recipe\IngredientList()\Ingredient = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Grab the recipe instructions
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "ProcedureText"
      
      ;--------------------------------------------------------------------------------------------
      ; Trim all unwanted rubbish from the recipe instructions
      ;--------------------------------------------------------------------------------------------
      
      Recipe\Instructions.s + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode))) + #LF$
      
;      Recipe\Instructions.s = ReplaceString(Recipe\Instructions.s, "'", "''")
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf  NodeName.s = "RecipeReview"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s         = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s        = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s

            Case "Rating"
              
              Recipe\Reciperating.s = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf      
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeTip"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      NodeText.s             = ReplaceString(GetXMLNodeText(*CurrentNode), "'", "''")
      
      Recipe\Othercomments.s + RecipeCRLFSpaceTrim(XmlToText(NodeText.s)) + #LF$ + #LF$
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeNutrition"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s           = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s          = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Recipe\Nutritionaldata.s  + AttributeName.s + " = " + AttributeValue.s  + #LF$
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Check for recipe images in the XML
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "SourceImage" Or NodeName.s = "RecipeImage"    ; Image description node
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------

      If ExamineXMLAttributes(*CurrentNode)
        
        AddElement(Recipe\PictureList())
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName
              
            Case "FileName"
              
              Recipe\PictureList()\FileName         = AttributeValue.s                                                          ; This is the filename physically linked on disk
              
            Case "Description"
              
              Recipe\PictureList()\Description      = AttributeValue.s
              
            Case "OriginalFileName"
              
              Recipe\PictureList()\OriginalFileName = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Recipe\PictureList()\FileName <> #EmptyString
        
        FDXZPicturePath.s = Program\TemporaryDirectory + "Unpacked\Images\"                                                     ;  : Debug FDXZPicturePath.s + Recipe\PictureList()\FileName + " Size: "  + FileSize(FDXZPicturePath.s + Recipe\PictureList()\FileName)
        
        If FileSize(FDXZPicturePath.s + Recipe\PictureList()\FileName) <> -1
          
;           Recipe\PictureList()\PictureBuffer    = AllocateMemory(#PictureBufferSize)
;           
;           If Recipe\PictureList()\PictureBuffer = #Null
;             
;             Debug "Picture memory buffer allocation error!"
;             
;           EndIf
          
          Recipe\PictureList()\FileType         = GetExtensionPart(Recipe\PictureList()\OriginalFileName)
          
          If Len(Recipe\PictureList()\OriginalFileName) = 0                                                                     ; If the filename came back empty
            
            If Len(Recipe\PictureList()\FileName) = 0
              
              Recipe\PictureList()\FileName = "Dummy." + Recipe\PictureList()\FileType
              
            EndIf
            
            Recipe\PictureList()\OriginalFileName = Recipe\Recipetitle + "." + GetExtensionPart(Recipe\PictureList()\FileName)  ; Add the recipe title as the filename
            
          EndIf
          
          PictureFileIn.i = ReadFile(#PB_Any, FDXZPicturePath.s  + Recipe\PictureList()\FileName)
          
          If PictureFileIn.i <> 0
            
            FileInSize.i = Lof(PictureFileIn.i)
            
            If FileInSize.i <> 0
              
              Recipe\PictureList()\PictureBuffer    = AllocateMemory(FileInSize.i)
              
              ;               If Recipe\PictureList()\PictureBuffer = #Null
              ;                 
              ;                 Debug "Picture memory buffer allocation error!"
              ;                 
              ;               EndIf
              
              If Recipe\PictureList()\PictureBuffer <>  #Null
                
                If ReadData(PictureFileIn.i, Recipe\PictureList()\PictureBuffer, FileInSize.i) = FileInSize.i
                  
                  Recipe\PictureList()\PictureBufferPtr = FileInSize.i
                  
                Else
                  
                  Debug "We could not read the picture for " + FDXZPicturePath.s  + Recipe\PictureList()\FileName
                  
                EndIf
                
              Else
                
                Debug "We got no picture buffer storage data for " + FDXZPicturePath.s  + Recipe\PictureList()\FileName
                
              EndIf
              
            Else
              
              Debug "We got no picture data for " + FDXZPicturePath.s  + Recipe\PictureList()\FileName
              
            EndIf
            
            CloseFile(PictureFileIn.i)
            
          Else
            
            Debug "We did not get a picture handle for "  + FDXZPicturePath.s  + Recipe\PictureList()\FileName
            
          EndIf
          
        Else
          
          Debug "The recipe picture file could not be found at: " + FDXZPicturePath.s
          
        EndIf
        
      Else
        
        Debug "There was no picture name for this recipe"
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    While *ChildNode <> 0
      
      ImportFoodDataExchangeCompressedFile_XML(ImportFileName, DataFileName.s,  *ChildNode, CurrentSublevel + 1)
      
      *ChildNode = NextXMLNode(*ChildNode)
      
    Wend

    ;----------------------------------------------------------------------------------------------
    ; Try to save the last one
    ;----------------------------------------------------------------------------------------------
    
    If *ChildNode = 0 And CurrentSublevel = 0
      
      ;Debug Str(*ChildNode) + " --- " + Str(CurrentSublevel)
      
      If Len(Recipe\Recipetitle)
        
        If Len(Recipe\Instructions.s)
          
          Recipe\Instructions.s = ReplaceString(Recipe\Instructions.s, "'", "''")
          
        EndIf
        
        SaveImportRecipe(ImportFileName.s)  ; Save the filled fields of the recipe structure
        
        ClearRecipeStructure()
        
      EndIf
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 430
; FirstLine = 408
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP
; CompileSourceDirectory